# Print an optspec for argparse to handle cmd's options that are independent of any subcommand.
function __fish_shh_global_optspecs
	string join \n h/help V/version
end

function __fish_shh_needs_command
	# Figure out if the current invocation already has a command.
	set -l cmd (commandline -opc)
	set -e cmd[1]
	argparse -s (__fish_shh_global_optspecs) -- $cmd 2>/dev/null
	or return
	if set -q argv[1]
		# Also print the command, so this can be used to figure out what it is.
		echo $argv[1]
		return 1
	end
	return 0
end

function __fish_shh_using_subcommand
	set -l cmd (__fish_shh_needs_command)
	test -z "$cmd"
	and return 1
	contains -- $cmd[1] $argv
end

complete -c shh -n "__fish_shh_needs_command" -s h -l help -d 'Print help'
complete -c shh -n "__fish_shh_needs_command" -s V -l version -d 'Print version'
complete -c shh -n "__fish_shh_needs_command" -f -a "run" -d 'Run a program to profile its behavior'
complete -c shh -n "__fish_shh_needs_command" -f -a "merge-profile-data" -d 'Merge profile data from previous runs to generate systemd options'
complete -c shh -n "__fish_shh_needs_command" -f -a "service" -d 'Act on a systemd service unit'
complete -c shh -n "__fish_shh_needs_command" -f -a "list-systemd-options" -d 'Dump markdown formatted list of supported systemd options'
complete -c shh -n "__fish_shh_needs_command" -f -a "gen-man-pages" -d 'Generate man pages'
complete -c shh -n "__fish_shh_needs_command" -f -a "gen-shell-complete" -d 'Generate shell completion'
complete -c shh -n "__fish_shh_needs_command" -f -a "help" -d 'Print this message or the help of the given subcommand(s)'
complete -c shh -n "__fish_shh_using_subcommand run" -s i -l instance -d 'Systemd instance of the service ("system" or "user" for per-user instances of the service manager)' -r -f -a "system\t''
user\t''"
complete -c shh -n "__fish_shh_using_subcommand run" -s m -l mode -d 'How hard we should harden' -r -f -a "safe\t'Only generate hardening options if they have a very low risk of breaking things'
aggressive\t'Will harden further and prevent circumventing restrictions of some options, but may increase the risk of breaking services'"
complete -c shh -n "__fish_shh_using_subcommand run" -l merge-paths-threshold -d 'When using whitelist-based filesystem hardening, if path whitelist is longer than this value, try to merge paths with the same parent' -r
complete -c shh -n "__fish_shh_using_subcommand run" -l systemd-options -d 'Disable all systemd options except these (case sensitive). Other options may be generated when mutating these options to make them compatible with profiling data. For testing only' -r
complete -c shh -n "__fish_shh_using_subcommand run" -s p -l profile-data-path -d 'Generate profile data file to be merged with others instead of generating systemd options directly' -r -F
complete -c shh -n "__fish_shh_using_subcommand run" -s l -l strace-log-path -d 'Log strace output to this file. Only use for debugging: this will slow down processing, and may generate a huge file' -r -F
complete -c shh -n "__fish_shh_using_subcommand run" -s f -l network-firewalling -d 'Enable advanced network firewalling. Only use this if you know that the network addresses and ports of local system and remote peers will not change'
complete -c shh -n "__fish_shh_using_subcommand run" -s w -l filesystem-whitelisting -d 'Enable whitelist-based filesystem hardening. Only use this if you know that the paths accessed by the service will not change'
complete -c shh -n "__fish_shh_using_subcommand run" -s h -l help -d 'Print help (see more with \'--help\')'
complete -c shh -n "__fish_shh_using_subcommand merge-profile-data" -s i -l instance -d 'Systemd instance of the service ("system" or "user" for per-user instances of the service manager)' -r -f -a "system\t''
user\t''"
complete -c shh -n "__fish_shh_using_subcommand merge-profile-data" -s m -l mode -d 'How hard we should harden' -r -f -a "safe\t'Only generate hardening options if they have a very low risk of breaking things'
aggressive\t'Will harden further and prevent circumventing restrictions of some options, but may increase the risk of breaking services'"
complete -c shh -n "__fish_shh_using_subcommand merge-profile-data" -l merge-paths-threshold -d 'When using whitelist-based filesystem hardening, if path whitelist is longer than this value, try to merge paths with the same parent' -r
complete -c shh -n "__fish_shh_using_subcommand merge-profile-data" -l systemd-options -d 'Disable all systemd options except these (case sensitive). Other options may be generated when mutating these options to make them compatible with profiling data. For testing only' -r
complete -c shh -n "__fish_shh_using_subcommand merge-profile-data" -s f -l network-firewalling -d 'Enable advanced network firewalling. Only use this if you know that the network addresses and ports of local system and remote peers will not change'
complete -c shh -n "__fish_shh_using_subcommand merge-profile-data" -s w -l filesystem-whitelisting -d 'Enable whitelist-based filesystem hardening. Only use this if you know that the paths accessed by the service will not change'
complete -c shh -n "__fish_shh_using_subcommand merge-profile-data" -s h -l help -d 'Print help (see more with \'--help\')'
complete -c shh -n "__fish_shh_using_subcommand service; and not __fish_seen_subcommand_from start-profile finish-profile reset help" -s h -l help -d 'Print help'
complete -c shh -n "__fish_shh_using_subcommand service; and not __fish_seen_subcommand_from start-profile finish-profile reset help" -f -a "start-profile" -d 'Add fragment config to service to profile its behavior'
complete -c shh -n "__fish_shh_using_subcommand service; and not __fish_seen_subcommand_from start-profile finish-profile reset help" -f -a "finish-profile" -d 'Get profiling result and remove fragment config from service'
complete -c shh -n "__fish_shh_using_subcommand service; and not __fish_seen_subcommand_from start-profile finish-profile reset help" -f -a "reset" -d 'Remove profiling and/or hardening config fragments, and restart service to restore its initial state'
complete -c shh -n "__fish_shh_using_subcommand service; and not __fish_seen_subcommand_from start-profile finish-profile reset help" -f -a "help" -d 'Print this message or the help of the given subcommand(s)'
complete -c shh -n "__fish_shh_using_subcommand service; and __fish_seen_subcommand_from start-profile" -s i -l instance -d 'Systemd instance of the service ("system" or "user" for per-user instances of the service manager)' -r -f -a "system\t''
user\t''"
complete -c shh -n "__fish_shh_using_subcommand service; and __fish_seen_subcommand_from start-profile" -s m -l mode -d 'How hard we should harden' -r -f -a "safe\t'Only generate hardening options if they have a very low risk of breaking things'
aggressive\t'Will harden further and prevent circumventing restrictions of some options, but may increase the risk of breaking services'"
complete -c shh -n "__fish_shh_using_subcommand service; and __fish_seen_subcommand_from start-profile" -l merge-paths-threshold -d 'When using whitelist-based filesystem hardening, if path whitelist is longer than this value, try to merge paths with the same parent' -r
complete -c shh -n "__fish_shh_using_subcommand service; and __fish_seen_subcommand_from start-profile" -l systemd-options -d 'Disable all systemd options except these (case sensitive). Other options may be generated when mutating these options to make them compatible with profiling data. For testing only' -r
complete -c shh -n "__fish_shh_using_subcommand service; and __fish_seen_subcommand_from start-profile" -s f -l network-firewalling -d 'Enable advanced network firewalling. Only use this if you know that the network addresses and ports of local system and remote peers will not change'
complete -c shh -n "__fish_shh_using_subcommand service; and __fish_seen_subcommand_from start-profile" -s w -l filesystem-whitelisting -d 'Enable whitelist-based filesystem hardening. Only use this if you know that the paths accessed by the service will not change'
complete -c shh -n "__fish_shh_using_subcommand service; and __fish_seen_subcommand_from start-profile" -s n -l no-restart -d 'Disable immediate service restart'
complete -c shh -n "__fish_shh_using_subcommand service; and __fish_seen_subcommand_from start-profile" -s h -l help -d 'Print help (see more with \'--help\')'
complete -c shh -n "__fish_shh_using_subcommand service; and __fish_seen_subcommand_from finish-profile" -s i -l instance -d 'Systemd instance of the service ("system" or "user" for per-user instances of the service manager)' -r -f -a "system\t''
user\t''"
complete -c shh -n "__fish_shh_using_subcommand service; and __fish_seen_subcommand_from finish-profile" -s a -l apply -d 'Automatically apply hardening config'
complete -c shh -n "__fish_shh_using_subcommand service; and __fish_seen_subcommand_from finish-profile" -s e -l edit -d 'Edit generated options before applying them'
complete -c shh -n "__fish_shh_using_subcommand service; and __fish_seen_subcommand_from finish-profile" -s n -l no-restart -d 'Disable immediate service restart'
complete -c shh -n "__fish_shh_using_subcommand service; and __fish_seen_subcommand_from finish-profile" -s h -l help -d 'Print help'
complete -c shh -n "__fish_shh_using_subcommand service; and __fish_seen_subcommand_from reset" -s i -l instance -d 'Systemd instance of the service ("system" or "user" for per-user instances of the service manager)' -r -f -a "system\t''
user\t''"
complete -c shh -n "__fish_shh_using_subcommand service; and __fish_seen_subcommand_from reset" -s h -l help -d 'Print help'
complete -c shh -n "__fish_shh_using_subcommand service; and __fish_seen_subcommand_from help" -f -a "start-profile" -d 'Add fragment config to service to profile its behavior'
complete -c shh -n "__fish_shh_using_subcommand service; and __fish_seen_subcommand_from help" -f -a "finish-profile" -d 'Get profiling result and remove fragment config from service'
complete -c shh -n "__fish_shh_using_subcommand service; and __fish_seen_subcommand_from help" -f -a "reset" -d 'Remove profiling and/or hardening config fragments, and restart service to restore its initial state'
complete -c shh -n "__fish_shh_using_subcommand service; and __fish_seen_subcommand_from help" -f -a "help" -d 'Print this message or the help of the given subcommand(s)'
complete -c shh -n "__fish_shh_using_subcommand list-systemd-options" -s h -l help -d 'Print help'
complete -c shh -n "__fish_shh_using_subcommand gen-man-pages" -s h -l help -d 'Print help'
complete -c shh -n "__fish_shh_using_subcommand gen-shell-complete" -s s -l shell -d 'Shell to generate for, leave empty for all' -r -f -a "bash\t''
elvish\t''
fish\t''
powershell\t''
zsh\t''"
complete -c shh -n "__fish_shh_using_subcommand gen-shell-complete" -s h -l help -d 'Print help'
complete -c shh -n "__fish_shh_using_subcommand help; and not __fish_seen_subcommand_from run merge-profile-data service list-systemd-options gen-man-pages gen-shell-complete help" -f -a "run" -d 'Run a program to profile its behavior'
complete -c shh -n "__fish_shh_using_subcommand help; and not __fish_seen_subcommand_from run merge-profile-data service list-systemd-options gen-man-pages gen-shell-complete help" -f -a "merge-profile-data" -d 'Merge profile data from previous runs to generate systemd options'
complete -c shh -n "__fish_shh_using_subcommand help; and not __fish_seen_subcommand_from run merge-profile-data service list-systemd-options gen-man-pages gen-shell-complete help" -f -a "service" -d 'Act on a systemd service unit'
complete -c shh -n "__fish_shh_using_subcommand help; and not __fish_seen_subcommand_from run merge-profile-data service list-systemd-options gen-man-pages gen-shell-complete help" -f -a "list-systemd-options" -d 'Dump markdown formatted list of supported systemd options'
complete -c shh -n "__fish_shh_using_subcommand help; and not __fish_seen_subcommand_from run merge-profile-data service list-systemd-options gen-man-pages gen-shell-complete help" -f -a "gen-man-pages" -d 'Generate man pages'
complete -c shh -n "__fish_shh_using_subcommand help; and not __fish_seen_subcommand_from run merge-profile-data service list-systemd-options gen-man-pages gen-shell-complete help" -f -a "gen-shell-complete" -d 'Generate shell completion'
complete -c shh -n "__fish_shh_using_subcommand help; and not __fish_seen_subcommand_from run merge-profile-data service list-systemd-options gen-man-pages gen-shell-complete help" -f -a "help" -d 'Print this message or the help of the given subcommand(s)'
complete -c shh -n "__fish_shh_using_subcommand help; and __fish_seen_subcommand_from service" -f -a "start-profile" -d 'Add fragment config to service to profile its behavior'
complete -c shh -n "__fish_shh_using_subcommand help; and __fish_seen_subcommand_from service" -f -a "finish-profile" -d 'Get profiling result and remove fragment config from service'
complete -c shh -n "__fish_shh_using_subcommand help; and __fish_seen_subcommand_from service" -f -a "reset" -d 'Remove profiling and/or hardening config fragments, and restart service to restore its initial state'
